@@ -57,8 +57,6 @@ struct _TangleActorPrivate {
 	gulong paint_event_handler_id;
 	gulong pick_event_handler_id;
 
-	GHashTable* animations;
-
 	TangleSpacing margin;
 	gfloat max_width;
 	gfloat max_height;
@@ -80,9 +78,6 @@ struct _TangleActorPrivate {
 	TangleAction* show_completed_action;
 	TangleAction* transition_completed_action;
 	
-	GList* ancestor_events;
-	GList* descendant_events;
-	
 	ClutterTimeline* hiding_timeline;
 	
 	GHashTable* template_parameters;
@@ -94,8 +89,10 @@ struct _TangleActorPrivate {
 	guint max_height_set : 1;
 	guint aspect_ratio_set : 1;
 	guint interacting : 1;
+	guint dragging : 1;
+	guint ancestor_interacting : 1;
+	guint ancestor_dragging : 1;
 	guint interactive_size_multiplier_set : 1;
-	guint propagating_events : 1;
 	guint hide_all : 1;
 	guint destroy : 1;
 };
@@ -120,6 +117,7 @@ enum {
 	PROP_SQUINT_X,
 	PROP_SQUINT_Y,
 	PROP_MARGIN,
+	PROP_MARGIN_SET,
 	PROP_MAX_WIDTH,
 	PROP_MAX_WIDTH_SET,
 	PROP_MAX_HEIGHT,
@@ -131,6 +129,9 @@ enum {
 	PROP_ALIGNMENT_MODE,
 	PROP_ALIGNED_ALLOCATION,
 	PROP_INTERACTING,
+	PROP_DRAGGING,
+	PROP_ANCESTOR_INTERACTING,
+	PROP_ANCESTOR_DRAGGING,
 	PROP_INTERACTIVE_SIZE_MULTIPLIER,
 	PROP_INTERACTIVE_SIZE_MULTIPLIER_SET,
 	PROP_STYLE,
@@ -143,7 +144,8 @@ enum {
 	PROP_HIDE_ACTION,
 	PROP_SHOW_ACTION,
 	PROP_SHOW_COMPLETED_ACTION,
-	PROP_TRANSITION_COMPLETED_ACTION
+	PROP_TRANSITION_COMPLETED_ACTION,
+	PROP_ACTION
 };
 
 enum {
@@ -154,75 +156,52 @@ enum {
 };
 
 static ClutterTimeline* animate_transition(TangleActor* actor, ClutterActorBox* from_allocation, ClutterActorBox* to_allocation);
-static gboolean is_value_user_data(gpointer key, gpointer value, gpointer user_data);
-static void on_animation_completed(ClutterAnimation* animation, gpointer user_data);
 static void set_max_width(TangleActor* actor, gfloat max_width);
 static void set_max_height(TangleActor* actor, gfloat max_height);
 static void set_scale_centers_by_gravity(TangleActor* actor);
 static void set_picking_scale_gravity(TangleActor* actor, ClutterGravity gravity);
 static void set_transition_scale_gravity(TangleActor* actor, ClutterGravity gravity);
-static gboolean handle_queued_events(TangleActor* actor);
 static void unset_transition_scale_children(TangleActor* actor);
 static void on_hiding_timeline_completed(TangleActor* actor);
-static void setup_animation_valist(ClutterAnimation* animation, TangleActor* actor, const gchar* first_property_name, va_list args);
-static void setup_animation_array(ClutterAnimation* animation, TangleActor* actor, gint n_properties, const gchar* const property_names[], const GValue* property_values);
 static void complete_timeline(ClutterTimeline* timeline);
+static gboolean parse_spacing_from_json_array(TangleSpacing* spacing, JsonArray* array);
+static gboolean parse_spacing_from_json_object(TangleSpacing* spacing, JsonObject* object);
+static void update_ancestor_interacting_and_dragging(TangleActor* actor, TangleActor* parent);
+static void on_notify_interacting_or_dragging(GObject* object, GParamSpec* param_spec, gpointer user_data);
 
 static guint signals[LAST_SIGNAL] = { 0 };
 static ClutterScriptableIface* parent_scriptable_iface = NULL;
 
-ClutterAnimation* tangle_actor_animate(TangleActor* actor, gulong mode, guint duration, const gchar* first_property_name, ...) {
+ClutterAnimation* tangle_actor_animate(TangleActor* actor, gulong mode, guint duration, ...) {
 	ClutterAnimation* animation;
 	va_list args;
 	
-	animation = clutter_animation_new();
-	clutter_animation_set_mode(animation, mode);
-	clutter_animation_set_duration(animation, duration);
-
-	va_start(args, first_property_name);
-	setup_animation_valist(animation, actor, first_property_name, args);
+	va_start(args, duration);
+	animation = tangle_object_animate_valist(G_OBJECT(actor), mode, duration, args);
 	va_end(args);
-
+	
 	return animation;
 }
 
-ClutterAnimation* tangle_actor_animate_with_timeline(TangleActor* actor, gulong mode, ClutterTimeline* timeline, const gchar* first_property_name, ...) {
+ClutterAnimation* tangle_actor_animate_with_timeline(TangleActor* actor, gulong mode, ClutterTimeline* timeline, ...) {
 	ClutterAnimation* animation;
 	va_list args;
 	
-	animation = clutter_animation_new();
-	clutter_animation_set_mode(animation, mode);
-	clutter_animation_set_timeline(animation, timeline);
-
-	va_start(args, first_property_name);
-	setup_animation_valist(animation, actor, first_property_name, args);
+	va_start(args, timeline);
+	animation = tangle_object_animate_valist_with_timeline(G_OBJECT(actor), mode, timeline, args);
 	va_end(args);
 
 	return animation;
 }
 
 ClutterAnimation* tangle_actor_animatev(TangleActor* actor, gulong mode, guint duration, gint n_properties, const gchar* const property_names[], const GValue* property_values) {
-	ClutterAnimation* animation;
-	va_list args;
-	
-	animation = clutter_animation_new();
-	clutter_animation_set_mode(animation, mode);
-	clutter_animation_set_duration(animation, duration);
 
-	setup_animation_array(animation, actor, n_properties, property_names, property_values);
-
-	return animation;
+	return tangle_object_animatev(G_OBJECT(actor), mode, duration, n_properties, property_names, property_values);
 }
 
 gboolean tangle_actor_stop_animation(TangleActor* actor, ClutterAnimation* animation) {
-	gboolean retvalue = FALSE;
 	
-	if (g_hash_table_foreach_remove(actor->priv->animations, is_value_user_data, animation)) {
-		g_object_unref(animation);
-		retvalue = TRUE;
-	}
-	
-	return retvalue;
+	return tangle_object_stop_animation(G_OBJECT(actor), animation);
 }
 
 TangleTransition* tangle_actor_get_transition(TangleActor* actor) {
@@ -478,7 +457,7 @@ void tangle_actor_set_margin(TangleActor* actor, const TangleSpacing* spacing) {
 		actor->priv->aligned_allocation.x2 -= actor->priv->margin.right;
 		actor->priv->aligned_allocation.y2 -= actor->priv->margin.bottom;
 		g_object_notify(G_OBJECT(actor), "margin");
-		g_object_notify(G_OBJECT(actor), "aligned-allocation");
+/* TODO		g_object_notify(G_OBJECT(actor), "aligned-allocation"); */
 		tangle_actor_set_margin_set(actor, TRUE);
 
 		g_object_thaw_notify(G_OBJECT(actor));
@@ -596,6 +575,28 @@ void tangle_actor_set_interacting(TangleActor* actor, gboolean interacting) {
 	}
 }
 
+gboolean tangle_actor_get_dragging(TangleActor* actor) {
+
+	return actor->priv->dragging;
+}
+
+void tangle_actor_set_dragging(TangleActor* actor, gboolean dragging) {
+	if (actor->priv->dragging != dragging) {
+		actor->priv->dragging = dragging;
+		g_object_notify(G_OBJECT(actor), "dragging");
+	}
+}
+
+gboolean tangle_actor_get_ancestor_interacting(TangleActor* actor) {
+
+	return actor->priv->ancestor_interacting;
+}
+
+gboolean tangle_actor_get_ancestor_dragging(TangleActor* actor) {
+
+	return actor->priv->ancestor_dragging;
+}
+
 void tangle_actor_get_preferred_width(TangleActor* actor, gfloat for_height, gboolean interacting, gfloat* min_width_p, gfloat* natural_width_p, gfloat* max_width_p) {
 	gfloat min_width;
 	gboolean min_width_set;
@@ -800,33 +801,15 @@ void tangle_actor_set_interactive_size_multiplier_set(TangleActor* actor, gboole
 	}
 }
 
-void tangle_actor_send_event(TangleActor* actor, TangleEvent* event) {
-	tangle_actor_handle_ancestor_event(actor, event);
-	tangle_actor_handle_descendant_event(actor, event);
-}
+GType tangle_spacing_get_type(void) {
+	static GType type = 0;
 
-void tangle_actor_handle_ancestor_event(TangleActor* actor, TangleEvent* event) {
-	if (actor->priv->propagating_events) {
-		actor->priv->ancestor_events = g_list_prepend(actor->priv->ancestor_events, tangle_event_copy(event));
-	} else {
-		actor->priv->propagating_events = TRUE;
-		TANGLE_ACTOR_GET_CLASS(actor)->handle_ancestor_event(actor, event);
-		while (!handle_queued_events(actor)) {
-		}
-		actor->priv->propagating_events = FALSE;
-	}
-}
+	if (!type)
+		type = g_boxed_type_register_static("TangleSpacing",
+		                                    (GBoxedCopyFunc)tangle_spacing_copy,
+		                                    (GBoxedFreeFunc)tangle_spacing_free);
 
-void tangle_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event) {
-	if (actor->priv->propagating_events) {
-		actor->priv->descendant_events = g_list_prepend(actor->priv->descendant_events, tangle_event_copy(event));
-	} else {
-		actor->priv->propagating_events = TRUE;
-		TANGLE_ACTOR_GET_CLASS(actor)->handle_descendant_event(actor, event);
-		while (!handle_queued_events(actor)) {
-		}
-		actor->priv->propagating_events = FALSE;
-	}
+	return type;
 }
 
 TangleSpacing* tangle_spacing_new(gfloat top, gfloat right, gfloat bottom, gfloat left) {
@@ -862,18 +845,40 @@ gboolean tangle_spacing_equal(const TangleSpacing* spacing_a, const TangleSpacin
 	       spacing_a->left == spacing_b->left;
 }
 
-static gboolean is_value_user_data(gpointer key, gpointer value, gpointer user_data) {
-
-	return value == user_data;
+gboolean tangle_spacing_parse_from_json(TangleSpacing* spacing, JsonNode* node) {
+	gboolean retvalue = FALSE;
+	
+	switch (JSON_NODE_TYPE(node)) {
+		case JSON_NODE_ARRAY:
+			retvalue = parse_spacing_from_json_array(spacing, json_node_get_array(node));
+			break;
+		case JSON_NODE_OBJECT:
+			retvalue = parse_spacing_from_json_object(spacing, json_node_get_object(node));
+			break;
+		default:
+			g_warning("Failed to parse TangleSpacing from JSON node that was not an array or an object, skipping.\n");
+			break;
+	}
+	
+	return retvalue;
 }
 
-static void on_animation_completed(ClutterAnimation* animation, gpointer user_data) {
-	TangleActor* actor;
+ClutterAction* tangle_actor_get_action_by_type(ClutterActor* actor, GType type) {
+	ClutterAction* action = NULL;
+	GList* list;
+	GList* list_item;
+	
+	g_return_val_if_fail(g_type_is_a(type, CLUTTER_TYPE_ACTION), NULL);
+	
+	list = clutter_actor_get_actions(actor);
+	for (list_item = list; !action && list_item; list_item = list_item->next) {
+		if (g_type_is_a(G_OBJECT_TYPE(list_item->data), type)) {
+			action = CLUTTER_ACTION(list_item->data);
+		}
+	}
+	g_list_free(list);
 
-	actor = TANGLE_ACTOR(user_data);
-
-	g_hash_table_foreach_remove(actor->priv->animations, is_value_user_data, animation);
-	g_object_unref(animation);
+	return action;
 }
 
 static void set_max_width_set(TangleActor* actor, gboolean is_set) {
@@ -1185,6 +1190,26 @@ static gboolean tangle_actor_scroll_event(ClutterActor* clutter_actor, ClutterSc
 	return FALSE;
 }
 
+static void tangle_actor_parent_set(ClutterActor* actor, ClutterActor* old_parent) {
+	ClutterActor* parent;
+	
+	if (old_parent && TANGLE_IS_ACTOR(old_parent)) {
+		g_signal_handlers_disconnect_by_func(old_parent, G_CALLBACK(on_notify_interacting_or_dragging), actor);
+	}
+	
+	parent = clutter_actor_get_parent(actor);
+	if (parent && TANGLE_IS_ACTOR(parent)) {
+		update_ancestor_interacting_and_dragging(TANGLE_ACTOR(actor), TANGLE_ACTOR(parent));
+
+		g_signal_connect(parent, "notify::interacting", G_CALLBACK(on_notify_interacting_or_dragging), actor);
+		g_signal_connect(parent, "notify::ancestor-interacting", G_CALLBACK(on_notify_interacting_or_dragging), actor);
+		g_signal_connect(parent, "notify::dragging", G_CALLBACK(on_notify_interacting_or_dragging), actor);
+		g_signal_connect(parent, "notify::ancestor-dragging", G_CALLBACK(on_notify_interacting_or_dragging), actor);
+	} else {
+		update_ancestor_interacting_and_dragging(TANGLE_ACTOR(actor), NULL);
+	}	
+}
+
 static ClutterTimeline* tangle_actor_animate_transition(TangleActor* actor, ClutterActorBox* current_box, ClutterActorBox* new_box) {
 	ClutterTimeline* timeline = NULL;
 	
@@ -1243,19 +1268,6 @@ static void tangle_actor_pick_event_handler(ClutterActor* self, gpointer user_da
 	}
 }
 
-static void tangle_actor_real_handle_ancestor_event(TangleActor* actor, TangleEvent* event) {
-}
-
-static void tangle_actor_real_handle_descendant_event(TangleActor* actor, TangleEvent* event) {
-	ClutterActor* parent;
-
-	parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor));
-	if (parent && TANGLE_IS_ACTOR(parent)) {
-		tangle_actor_handle_descendant_event(TANGLE_ACTOR(parent), event);
-	}
-	
-}
-
 #define SET_ACTION(a) \
 	if (actor->priv->a) { \
 		g_object_unref(actor->priv->a); \
@@ -1335,6 +1347,12 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 			actor->priv->squint_y = g_value_get_float(value);
 			clutter_actor_queue_redraw(CLUTTER_ACTOR(actor));
 			break;
+		case PROP_MARGIN:
+			tangle_actor_set_margin(actor, TANGLE_SPACING(g_value_get_boxed(value)));
+			break;
+		case PROP_MARGIN_SET:
+			tangle_actor_set_margin_set(actor, g_value_get_boolean(value));
+			break;
 		case PROP_ASPECT_RATIO:
 			tangle_actor_set_aspect_ratio(actor, g_value_get_double(value));
 			break;
@@ -1344,6 +1362,9 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 		case PROP_INTERACTING:
 			tangle_actor_set_interacting(actor, g_value_get_boolean(value));
 			break;
+		case PROP_DRAGGING:
+			tangle_actor_set_dragging(actor, g_value_get_boolean(value));
+			break;
 		case PROP_INTERACTIVE_SIZE_MULTIPLIER:
 			tangle_actor_set_interactive_size_multiplier(actor, g_value_get_double(value));
 			break;
@@ -1380,6 +1401,9 @@ static void tangle_actor_set_property(GObject* object, guint prop_id, const GVal
 		case PROP_TRANSITION_COMPLETED_ACTION:
 			SET_ACTION(transition_completed_action);
 			break;
+		case PROP_ACTION:
+			clutter_actor_add_action(CLUTTER_ACTOR(actor), CLUTTER_ACTION(g_value_get_object(value)));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -1439,6 +1463,12 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 		case PROP_SQUINT_Y:
 			g_value_set_float(value, actor->priv->squint_y);
 			break;
+		case PROP_MARGIN:
+			g_value_set_boxed(value, &actor->priv->margin);
+			break;
+		case PROP_MARGIN_SET:
+			g_value_set_boolean(value, actor->priv->margin_set);
+			break;
 		case PROP_ASPECT_RATIO:
 			g_value_set_double(value, actor->priv->aspect_ratio);
 			break;
@@ -1448,6 +1478,15 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 		case PROP_INTERACTING:
 			g_value_set_boolean(value, actor->priv->interacting);
 			break;
+		case PROP_DRAGGING:
+			g_value_set_boolean(value, actor->priv->dragging);
+			break;
+		case PROP_ANCESTOR_INTERACTING:
+			g_value_set_boolean(value, actor->priv->ancestor_interacting);
+			break;
+		case PROP_ANCESTOR_DRAGGING:
+			g_value_set_boolean(value, actor->priv->ancestor_dragging);
+			break;
 		case PROP_INTERACTIVE_SIZE_MULTIPLIER:
 			g_value_set_double(value, actor->priv->interactive_size_multiplier);
 			break;
@@ -1492,13 +1531,10 @@ static void tangle_actor_get_property(GObject* object, guint prop_id, GValue* va
 
 static void tangle_actor_dispose(GObject* object) {
 	TangleActor* actor;
+	ClutterActor* parent;
 	
 	actor = TANGLE_ACTOR(object);
 
-	if (actor->priv->animations) {
-		g_hash_table_destroy(actor->priv->animations);
-		actor->priv->animations = NULL;
-	}
 
 	TANGLE_UNREF_AND_NULLIFY_OBJECT(actor->priv->button_press_action);
 	TANGLE_UNREF_AND_NULLIFY_OBJECT(actor->priv->button_release_action);
@@ -1511,17 +1547,6 @@ static void tangle_actor_dispose(GObject* object) {
 	TANGLE_UNREF_AND_NULLIFY_OBJECT(actor->priv->show_completed_action);
 	TANGLE_UNREF_AND_NULLIFY_OBJECT(actor->priv->transition_completed_action);
 
-	if (actor->priv->ancestor_events) {
-		g_list_foreach(actor->priv->ancestor_events, (GFunc)tangle_event_free, NULL);
-		g_list_free(actor->priv->ancestor_events);
-		actor->priv->ancestor_events = NULL;
-	}
-	if (actor->priv->descendant_events) {
-		g_list_foreach(actor->priv->descendant_events, (GFunc)tangle_event_free, NULL);
-		g_list_free(actor->priv->descendant_events);
-		actor->priv->descendant_events = NULL;
-	}
-
 	if (actor->priv->hiding_timeline) {
 		complete_timeline(actor->priv->hiding_timeline);
 		g_object_unref(actor->priv->hiding_timeline);
@@ -1533,6 +1558,11 @@ static void tangle_actor_dispose(GObject* object) {
 		actor->priv->template_parameters = NULL;
 	}
 	
+	parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor));
+	if (parent && TANGLE_IS_ACTOR(parent)) {
+		g_signal_handlers_disconnect_by_func(parent, G_CALLBACK(on_notify_interacting_or_dragging), actor);
+	}
+	
 	G_OBJECT_CLASS (tangle_actor_parent_class)->dispose (object);
 }
 
@@ -1559,12 +1589,11 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	clutter_actor_class->leave_event = tangle_actor_leave_event;
 	clutter_actor_class->motion_event = tangle_actor_motion_event;
 	clutter_actor_class->scroll_event = tangle_actor_scroll_event;
+	clutter_actor_class->parent_set = tangle_actor_parent_set;
 	
 	actor_class->animate_transition = tangle_actor_animate_transition;
 	actor_class->transition_completed = tangle_actor_transition_completed;
 	actor_class->show_completed = tangle_actor_show_completed;
-	actor_class->handle_ancestor_event = tangle_actor_real_handle_ancestor_event;
-	actor_class->handle_descendant_event = tangle_actor_real_handle_descendant_event;
 
 	/**
 	 * TangleActor:template:
@@ -1746,6 +1775,28 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	                                                   "The vertical squint (drawing offset)",
 	                                                   -G_MAXFLOAT, G_MAXFLOAT, 0.0,
 	                                                   G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:margin:
+	 *
+	 * The marginals around the actor.
+	 */
+	g_object_class_install_property(gobject_class, PROP_MARGIN,
+	                                g_param_spec_boxed("margin",
+	                                                    "Margin",
+	                                                    "The marginals around the actor",
+	                                                    TANGLE_TYPE_SPACING,
+	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:margin-set:
+	 *
+	 * Whether the :margin is set.
+	 */
+	g_object_class_install_property(gobject_class, PROP_MARGIN_SET,
+	                                g_param_spec_boolean("margin-set",
+	                                                    "Margin set",
+	                                                    "Whether the margin is set",
+	                                                    FALSE,
+	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleActor:aspect-ratio:
 	 */
@@ -1770,9 +1821,36 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	g_object_class_install_property(gobject_class, PROP_INTERACTING,
 	                                g_param_spec_boolean("interacting",
 	                                "Interacting",
-	                                "Whether the widget is interacting with an user or not",
+	                                "Whether the actor is interacting with an user or not",
 	                                FALSE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:dragging:
+	 */
+	g_object_class_install_property(gobject_class, PROP_DRAGGING,
+	                                g_param_spec_boolean("dragging",
+	                                "Dragging",
+	                                "Whether the actor is under dragging",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:ancestor-interacting:
+	 */
+	g_object_class_install_property(gobject_class, PROP_ANCESTOR_INTERACTING,
+	                                g_param_spec_boolean("ancestor-interacting",
+	                                "Ancestor interacting",
+	                                "Whether an ancestor of this actor is interacting with an user or not",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:ancestor-dragging:
+	 */
+	g_object_class_install_property(gobject_class, PROP_ANCESTOR_DRAGGING,
+	                                g_param_spec_boolean("ancestor-dragging",
+	                                "Ancestor Dragging",
+	                                "Whether an ancestor of this actor is under dragging",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleWidget:interactive-multiplier:
 	 */
@@ -1881,6 +1959,17 @@ static void tangle_actor_class_init(TangleActorClass* actor_class) {
 	                                "The action that correspond to the transition completed signal",
 	                                TANGLE_TYPE_ACTION,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:action:
+	 *
+	 * Adds a new #ClutterAction into the actor.
+	 */
+	g_object_class_install_property(gobject_class, PROP_ACTION,
+	                                g_param_spec_object("action",
+	                                "Action",
+	                                "Adds a new action into the actor",
+	                                CLUTTER_TYPE_ACTION,
+	                                G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 
 	/**
@@ -1938,8 +2027,6 @@ static void tangle_actor_init(TangleActor* actor) {
 	actor->priv->alignment_x = 0.5;
 	actor->priv->alignment_y = 0.5;
 
-	actor->priv->animations = g_hash_table_new_full(g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)g_object_unref);
-
 	actor->priv->paint_event_handler_id = g_signal_connect(actor, "paint", G_CALLBACK(tangle_actor_paint_event_handler), NULL);
 	actor->priv->pick_event_handler_id = g_signal_connect(actor, "pick", G_CALLBACK(tangle_actor_pick_event_handler), NULL);
 }
@@ -2095,28 +2182,6 @@ static void set_transition_scale_gravity(TangleActor* actor, ClutterGravity grav
 	}
 }
 
-static gboolean handle_queued_events(TangleActor* actor) {
-	gboolean queue_empty = TRUE;
-	GList* last;
-	
-	if (actor->priv->descendant_events) {
-		last = g_list_last(actor->priv->descendant_events);
-		TANGLE_ACTOR_GET_CLASS(actor)->handle_descendant_event(actor, TANGLE_EVENT(last->data));
-		tangle_event_free(TANGLE_EVENT(last->data));
-		actor->priv->descendant_events = g_list_delete_link(actor->priv->descendant_events, last);
-		queue_empty = FALSE;
-	}
-	if (actor->priv->ancestor_events) {
-		last = g_list_last(actor->priv->ancestor_events);
-		TANGLE_ACTOR_GET_CLASS(actor)->handle_ancestor_event(actor, TANGLE_EVENT(last->data));
-		tangle_event_free(TANGLE_EVENT(last->data));
-		actor->priv->ancestor_events = g_list_delete_link(actor->priv->ancestor_events, last);
-		queue_empty = FALSE;
-	}
-	
-	return queue_empty;
-}
-
 static void unset_transition_scale_children(TangleActor* actor) {
 	actor->priv->transition_scale_children = FALSE;
 }
@@ -2139,80 +2204,76 @@ static void on_hiding_timeline_completed(TangleActor* actor) {
 	actor->priv->hide_all = FALSE;
 }
 
-static void setup_animation_valist(ClutterAnimation* animation, TangleActor* actor, const gchar* first_property_name, va_list args) {
-	GObjectClass *klass;
-	const gchar* property_name;
-
-	clutter_animation_set_object(animation, G_OBJECT(actor));
-	g_signal_connect(animation, "completed", G_CALLBACK(on_animation_completed), G_OBJECT(actor));
-
-	klass = G_OBJECT_GET_CLASS(actor);
-
-	for (property_name = first_property_name; property_name; property_name = va_arg(args, gchar*)) {
-		GParamSpec *param_spec;
-		GValue value = { 0 };
-		gchar* error;
-		
-		if ((param_spec = g_object_class_find_property(klass, property_name))) {
-			g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(param_spec));
-			error = NULL;
-			G_VALUE_COLLECT(&value, args, 0, &error);
-			if (!error) {
-				ClutterAnimation* existing_animation;
-				
-				if ((existing_animation = g_hash_table_lookup(actor->priv->animations, property_name))) {
-					clutter_animation_unbind_property(existing_animation, property_name);
-					g_hash_table_remove(actor->priv->animations, property_name);
-				}
-				clutter_animation_bind(animation, property_name, &value);
-				g_hash_table_insert(actor->priv->animations, g_strdup(property_name), g_object_ref(animation)); // gobject_ref_sink does not work with 0.9.2!
-			} else {
-				g_warning("%s: %s", G_STRLOC, error);
-				g_free(error);
-				break;
-			}
-			g_value_unset(&value);
-		} else {
-			g_warning ("Cannot bind property '%s'.", property_name);
-			break;
-		}
-	}
-
-	clutter_timeline_start(clutter_animation_get_timeline(animation));
-}
-
-static void setup_animation_array(ClutterAnimation* animation, TangleActor* actor, gint n_properties, const gchar* const property_names[], const GValue* property_values) {
-	GObjectClass *klass;
-	gint i;
-
-	clutter_animation_set_object(animation, G_OBJECT(actor));
-	g_signal_connect(animation, "completed", G_CALLBACK(on_animation_completed), G_OBJECT(actor));
-
-	klass = G_OBJECT_GET_CLASS(actor);
-
-	for (i = 0; i < n_properties; i++) {
-		const gchar* property_name;
-		GParamSpec *param_spec;
-
-		property_name = property_names[i];	
-		if ((param_spec = g_object_class_find_property(klass, property_name))) {
-			ClutterAnimation* existing_animation;
-
-			if ((existing_animation = g_hash_table_lookup(actor->priv->animations, property_name))) {
-				clutter_animation_unbind_property(existing_animation, property_name);
-				g_hash_table_remove(actor->priv->animations, property_name);
-			}
-			clutter_animation_bind(animation, property_name, &property_values[i]);
-			g_hash_table_insert(actor->priv->animations, g_strdup(property_name), g_object_ref(animation)); // gobject_ref_sink does not work with 0.9.2!
-		} else {
-			g_warning ("Cannot bind property '%s'.", property_name);
-			break;
-		}
-	}
-
-	clutter_timeline_start(clutter_animation_get_timeline(animation));
-}
-
 static void complete_timeline(ClutterTimeline* timeline) {
 	clutter_timeline_skip(timeline, clutter_timeline_get_duration(timeline) - clutter_timeline_get_elapsed_time(timeline));
 }
+
+static gboolean parse_spacing_from_json_array(TangleSpacing* spacing, JsonArray* array) {
+	gboolean retvalue = FALSE;
+	
+	if (json_array_get_length(array) != 4) {
+		g_warning("Expected four elements in JSON array containing TangleSpacing, skipping.");
+	} else {
+		spacing->top = json_array_get_double_element(array, 0);
+		spacing->right = json_array_get_double_element(array, 1);
+		spacing->bottom = json_array_get_double_element(array, 2);
+		spacing->left = json_array_get_double_element(array, 3);
+		retvalue = TRUE;
+	}
+	
+	return retvalue;
+}
+
+static gboolean parse_spacing_from_json_object(TangleSpacing* spacing, JsonObject* object) {
+	if (json_object_has_member(object, "top")) {
+		spacing->top = json_object_get_double_member(object, "top");
+	} else {
+		spacing->top = 0.0;
+	}
+	if (json_object_has_member(object, "right")) {
+		spacing->right = json_object_get_double_member(object, "right");
+	} else {
+		spacing->right = 0.0;
+	}
+	if (json_object_has_member(object, "bottom")) {
+		spacing->bottom = json_object_get_double_member(object, "bottom");
+	} else {
+		spacing->bottom = 0.0;
+	}
+	if (json_object_has_member(object, "left")) {
+		spacing->left = json_object_get_double_member(object, "left");
+	} else {
+		spacing->left = 0.0;
+	}
+	
+	return TRUE;
+}
+
+static void update_ancestor_interacting_and_dragging(TangleActor* actor, TangleActor* parent) {
+	gboolean ancestor_interacting;
+	gboolean ancestor_dragging;
+	
+	if (parent && (tangle_actor_get_interacting(parent) || tangle_actor_get_ancestor_interacting(parent))) {
+		ancestor_interacting = TRUE;
+	} else {
+		ancestor_interacting = FALSE;
+	}
+	if (actor->priv->ancestor_interacting != ancestor_interacting) {
+		actor->priv->ancestor_interacting = ancestor_interacting;
+		g_object_notify(G_OBJECT(actor), "ancestor-interacting");
+	}
+
+	if (parent && (tangle_actor_get_dragging(parent) || tangle_actor_get_ancestor_dragging(parent))) {
+		ancestor_dragging = TRUE;
+	} else {
+		ancestor_dragging = FALSE;
+	}
+	if (actor->priv->ancestor_dragging != ancestor_dragging) {
+		actor->priv->ancestor_dragging = ancestor_dragging;
+		g_object_notify(G_OBJECT(actor), "ancestor-dragging");
+	}
+}
+
+static void on_notify_interacting_or_dragging(GObject* object, GParamSpec* param_spec, gpointer user_data) {
+	update_ancestor_interacting_and_dragging(TANGLE_ACTOR(user_data), TANGLE_ACTOR(object));
+}
