@@ -71,7 +71,7 @@ static const GEnumValue binding_direction_values[] = {
 static ClutterScriptableIface* parent_scriptable_iface = NULL;
 static GQuark quark_property_bindings = 0;
 
-static gulong connect_notify_signal(GObject* object, const gchar* property, GCallback callback, gpointer user_data);
+static gulong connect_notify_signal(GObject* object, GParamSpec* param_spec, GCallback callback, gpointer user_data);
 static void disconnect_notify_signals(TangleBinding* binding);
 static void on_notify_a(GObject* object, GParamSpec* param_spec, gpointer user_data);
 static void on_notify_b(GObject* object, GParamSpec* param_spec, gpointer user_data);
@@ -240,6 +240,8 @@ static void tangle_binding_get_property(GObject* object, guint prop_id, GValue*
 
 static void tangle_binding_constructed(GObject* object) {
 	TangleBinding* binding;
+	GParamSpec* param_specs[2];
+	gboolean writable[2] = { FALSE, FALSE };
 	GType value_type;
 	
 	binding = TANGLE_BINDING(object);
@@ -252,21 +254,35 @@ static void tangle_binding_constructed(GObject* object) {
 	g_object_weak_ref(binding->priv->objects[0], on_destroy_notify, binding);
 	g_object_weak_ref(binding->priv->objects[1], on_destroy_notify, binding);
 
-	/* TODO: read-only properties ?! */
-	
-	if ((binding->priv->notify_signal_handler_ids[0] = connect_notify_signal(binding->priv->objects[0], binding->priv->properties[0], G_CALLBACK(on_notify_a), binding))) {
-		binding->priv->direction |= TANGLE_BINDING_A_TO_B;
+	if (!(param_specs[0] = g_object_class_find_property(G_OBJECT_GET_CLASS(binding->priv->objects[0]), binding->priv->properties[0]))) {
+		g_warning("Object has no property called '%s'.", binding->priv->properties[0]);
 	}
-	if ((binding->priv->notify_signal_handler_ids[1] = connect_notify_signal(binding->priv->objects[1], binding->priv->properties[1], G_CALLBACK(on_notify_b), binding))) {
-		binding->priv->direction |= TANGLE_BINDING_B_TO_A;
+	if (!(param_specs[1] = g_object_class_find_property(G_OBJECT_GET_CLASS(binding->priv->objects[1]), binding->priv->properties[1]))) {
+		g_warning("Object has no property called '%s'.", binding->priv->properties[1]);
 	}
 	
-	if (binding->priv->direction == TANGLE_BINDING_NONE) {
-		g_warning("Both properties '%s' and '%s' were write only.", binding->priv->properties[0], binding->priv->properties[1]);
-	} else {
-		value_type = G_PARAM_SPEC_VALUE_TYPE(g_object_class_find_property(G_OBJECT_GET_CLASS(binding->priv->objects[0]), binding->priv->properties[0]));
-		g_value_init(&binding->priv->current_value[0], value_type);
-		g_value_init(&binding->priv->current_value[1], value_type);
+	if (param_specs[0] && param_specs[1]) {
+		if ((param_specs[0]->flags & G_PARAM_WRITABLE) && !(param_specs[0]->flags & G_PARAM_CONSTRUCT_ONLY)) {
+			writable[0] = TRUE;
+		}
+		if ((param_specs[1]->flags & G_PARAM_WRITABLE) && !(param_specs[1]->flags & G_PARAM_CONSTRUCT_ONLY)) {
+			writable[1] = TRUE;
+		}
+
+		if (param_specs[0] && writable[1] && (binding->priv->notify_signal_handler_ids[0] = connect_notify_signal(binding->priv->objects[0], param_specs[0], G_CALLBACK(on_notify_a), binding))) {
+			binding->priv->direction |= TANGLE_BINDING_A_TO_B;
+		}
+		if (param_specs[0] && writable[0] && (binding->priv->notify_signal_handler_ids[1] = connect_notify_signal(binding->priv->objects[1], param_specs[1], G_CALLBACK(on_notify_b), binding))) {
+			binding->priv->direction |= TANGLE_BINDING_B_TO_A;
+		}
+
+		if (binding->priv->direction == TANGLE_BINDING_NONE) {
+			g_warning("Both properties '%s' and '%s' were write only.", binding->priv->properties[0], binding->priv->properties[1]);
+		} else {
+			value_type = G_PARAM_SPEC_VALUE_TYPE(g_object_class_find_property(G_OBJECT_GET_CLASS(binding->priv->objects[0]), binding->priv->properties[0]));
+			g_value_init(&binding->priv->current_value[0], value_type);
+			g_value_init(&binding->priv->current_value[1], value_type);
+		}
 	}
 }
 
@@ -419,36 +435,31 @@ static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
 	iface->set_custom_property = tangle_binding_set_custom_property;
 }
 
-static gulong connect_notify_signal(GObject* object, const gchar* property, GCallback callback, gpointer user_data) {
+static gulong connect_notify_signal(GObject* object, GParamSpec* param_spec, GCallback callback, gpointer user_data) {
 	gulong handler_id = 0;
 	GHashTable* hash_table;
 	gchar* key;
 	GList* list;
-	GParamSpec* param_spec;
 	gchar* s;
 	
-	if (!(param_spec = g_object_class_find_property(G_OBJECT_GET_CLASS(object), property))) {
-		g_warning("Object has no property called '%s'.", property);
+	if (!(hash_table = (GHashTable*)g_object_get_qdata(object, quark_property_bindings))) {
+		hash_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_list_free);
+		g_object_set_qdata(object, quark_property_bindings, hash_table);
+	}
+
+	if (g_hash_table_lookup_extended(hash_table, param_spec->name, (gpointer*)&key, (gpointer*)&list)) {
+		g_hash_table_steal(hash_table, key);
 	} else {
-		if (!(hash_table = (GHashTable*)g_object_get_qdata(object, quark_property_bindings))) {
-			hash_table = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_list_free);
-			g_object_set_qdata(object, quark_property_bindings, hash_table);
-		}
-		
-		if (g_hash_table_lookup_extended(hash_table, property, (gpointer*)&key, (gpointer*)&list)) {
-			g_hash_table_steal(hash_table, key);
-		} else {
-			list = NULL;
-			key = g_strdup(property);
-		}
-		list = g_list_prepend(list, user_data);
-		g_hash_table_insert(hash_table, key, list);
-	
-		if (param_spec->flags & G_PARAM_READABLE) {
-			s = g_strconcat("notify::", property, NULL);
-			handler_id = g_signal_connect(object, s, callback, user_data);
-			g_free(s);
-		}
+		list = NULL;
+		key = g_strdup(param_spec->name);
+	}
+	list = g_list_prepend(list, user_data);
+	g_hash_table_insert(hash_table, key, list);
+
+	if (param_spec->flags & G_PARAM_READABLE) {
+		s = g_strconcat("notify::", param_spec->name, NULL);
+		handler_id = g_signal_connect(object, s, callback, user_data);
+		g_free(s);
 	}
 	
 	return handler_id;
