@@ -229,31 +229,6 @@ static gboolean tangle_draggable_actor_button_press_event(ClutterActor* actor, C
 	return FALSE;
 }
 
-static void tangle_draggable_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event) {
-	TangleDraggableActor* draggable_actor;
-	
-	draggable_actor = TANGLE_DRAGGABLE_ACTOR(actor);
-	
-	if (event->any.sender != actor) {
-		switch (event->type) {
-			case TANGLE_EVENT_INTERACTION_BEGAN:
-				draggable_actor->priv->descendant_interacting = TRUE;
-				if (event->interaction.dragging) {
-					draggable_actor->priv->descendant_dragging = TRUE;
-				}
-				break;
-			case TANGLE_EVENT_INTERACTION_ENDED:
-				draggable_actor->priv->descendant_interacting = FALSE;
-				draggable_actor->priv->descendant_dragging = FALSE;
-				break;
-			default:
-				break;
-		}
-	}
-
-	TANGLE_ACTOR_CLASS(tangle_draggable_actor_parent_class)->handle_descendant_event(actor, event);
-}
-
 static void tangle_draggable_actor_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	TangleDraggableActor* draggable_actor;
 	
@@ -331,8 +306,6 @@ static void tangle_draggable_actor_class_init(TangleDraggableActorClass* draggab
 	
 	clutter_actor_class->button_press_event = tangle_draggable_actor_button_press_event;
 
-	actor_class->handle_descendant_event = tangle_draggable_actor_handle_descendant_event;
-
 	draggable_actor_class->begin_dragging = tangle_draggable_actor_begin_dragging;
 	draggable_actor_class->do_dragging = tangle_draggable_actor_do_dragging;
 	draggable_actor_class->end_dragging = tangle_draggable_actor_end_dragging;
@@ -447,7 +420,6 @@ static gboolean on_captured_event(ClutterActor* actor, ClutterEvent* event, gpoi
 		if (draggable_actor->priv->dragging_effective) {
 			handle_droppable_actor(draggable_actor->priv->dragging);
 			if (event->type == CLUTTER_BUTTON_RELEASE) {
-				tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(draggable_actor), TRUE, TRUE, TRUE);
 			}
 			g_signal_emit(draggable_actor, signals[(event->type == CLUTTER_MOTION ? DO_DRAGGING : END_DRAGGING)], 0, draggable_actor->priv->dragging, &handled);
 			if (event->type == CLUTTER_BUTTON_RELEASE) {
@@ -466,7 +438,6 @@ static gboolean on_captured_event(ClutterActor* actor, ClutterEvent* event, gpoi
 				handle_droppable_actor(draggable_actor->priv->dragging);
 				g_signal_emit(draggable_actor, signals[BEGIN_DRAGGING], 0, draggable_actor->priv->dragging, &handled);	
 				if (handled) {
-					tangle_actor_send_interaction_began_event(TANGLE_ACTOR(draggable_actor), TRUE, TRUE, TRUE);
 				}
 			}
 			if (event->type == CLUTTER_BUTTON_RELEASE || !handled) {
