@@ -50,7 +50,7 @@ ClutterActor* tangle_grid_new(gfloat grid_spacing_x, gfloat grid_spacing_y) {
 
 ClutterActor* tangle_grid_new_with_line_color(gfloat grid_spacing_x, gfloat grid_spacing_y, ClutterColor* line_color) {
 
-	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, "line-color", line_color, NULL));
+	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_GRID, "grid-spacing-x", grid_spacing_x, "grid-spacing-y", grid_spacing_y, "line-color-x", line_color, "line-color-y", line_color, NULL));
 }
 
 gfloat tangle_grid_get_grid_spacing_x(TangleGrid* grid) {
