@@ -21,53 +21,21 @@ enum {
 };
 
 struct _TangleObjectPrivate {
-	ClutterScript* script;
-	TangleTemplate* template;
-	
-	gboolean template_applied;
+	GHashTable* template_parameters;
+	gboolean template_found;
 };
 
 static ClutterScriptableIface* parent_scriptable_iface = NULL;
 
-static gboolean check_template_parameters(gpointer user_data);
-
-gchar* tangle_object_lookup_filename(TangleObject* object, const gchar* filename) {
-
-	return _tangle_script_lookup_filename(object->priv->script, filename);
-}
-
-ClutterScript* tangle_object_get_script(TangleObject* object) {
-
-	return object->priv->script;
-}
-
-void _tangle_object_set_script(TangleObject* object, ClutterScript* script) {
-	g_return_if_fail(TANGLE_IS_OBJECT(object));
-	g_return_if_fail(CLUTTER_IS_SCRIPT(script));
-	g_return_if_fail(!object->priv->script || object->priv->script == script);
-	
-	if (!object->priv->script) {
-		object->priv->script = script;
-		g_object_add_weak_pointer(G_OBJECT(object->priv->script), (gpointer*)&object->priv->script);
-	}
-}
-
 static void tangle_object_set_property(GObject* gobject, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	TangleObject* object;
 	
-	object = TANGLE_OBJECT(object);
+	object = TANGLE_OBJECT(gobject);
 
 	switch (prop_id) {
 		case PROP_TEMPLATE:
 			if (g_value_get_object(value) != NULL) {
-				g_return_if_fail(!object->priv->template);
-				object->priv->template = TANGLE_TEMPLATE(g_value_get_object(value));
-				g_object_ref(object->priv->template);
-				if (tangle_template_has_parameters(object->priv->template)) {
-					g_idle_add_full(G_PRIORITY_HIGH, check_template_parameters, gobject, NULL);
-				} else {
-					tangle_template_apply_properties(object->priv->template, gobject);
-				}
+				tangle_template_apply_properties(TANGLE_TEMPLATE(g_value_get_object(value)), gobject);
 			}
 			break;
 		default:
@@ -79,7 +47,7 @@ static void tangle_object_set_property(GObject* gobject, guint prop_id, const GV
 static void tangle_object_get_property(GObject* gobject, guint prop_id, GValue* value, GParamSpec* pspec) {
         TangleObject* object;
 
-	object = TANGLE_OBJECT(object);
+	object = TANGLE_OBJECT(gobject);
 
         switch (prop_id) {
 	        default:
@@ -97,9 +65,9 @@ static void tangle_object_dispose(GObject* gobject) {
 	
 	object = TANGLE_OBJECT(gobject);
 	
-	if (object->priv->script) {
-		g_object_remove_weak_pointer(G_OBJECT(object->priv->script), (gpointer*)&object->priv->script);
-		object->priv->script = NULL;
+	if (object->priv->template_parameters) {
+		g_hash_table_destroy(object->priv->template_parameters);
+		object->priv->template_parameters = NULL;
 	}
 	
 	G_OBJECT_CLASS(tangle_object_parent_class)->dispose(gobject);
@@ -115,13 +83,15 @@ static void tangle_object_class_init(TangleObjectClass* object_class) {
 
 	/**
 	 * TangleObject:template:
+	 *
+	 * Applies the given #TangleTemplate to this object.
 	 */
 	g_object_class_install_property(gobject_class, PROP_TEMPLATE,
 	                                g_param_spec_object("template",
 	                                "Template",
-	                                "A template that is used to construct the properties of this object",
+	                                "Applies the given template to this object",
 	                                TANGLE_TYPE_TEMPLATE,
-	                                G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	                                G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 	g_type_class_add_private(gobject_class, sizeof (TangleObjectPrivate));
 }
@@ -135,13 +105,8 @@ static gboolean tangle_object_parse_custom_node(ClutterScriptable* scriptable, C
 	TangleObject* object;
 	
 	object = TANGLE_OBJECT(scriptable);
-	g_return_val_if_fail(!object->priv->script || object->priv->script == script, FALSE);
 
-	if (!object->priv->script) {
-		_tangle_object_set_script(object, script);
-	}
-
-	if (!(retvalue = _tangle_scriptable_parse_custom_node(scriptable, script, value, name, node, object->priv->template, &object->priv->template_applied)) &&
+	if (!(retvalue = _tangle_scriptable_parse_custom_node(scriptable, script, value, name, node, &object->priv->template_parameters, &object->priv->template_found)) &&
 	    parent_scriptable_iface->parse_custom_node) {
 		retvalue = parent_scriptable_iface->parse_custom_node(scriptable, script, value, name, node);
 	}
@@ -150,7 +115,11 @@ static gboolean tangle_object_parse_custom_node(ClutterScriptable* scriptable, C
 }
 
 static void tangle_object_set_custom_property(ClutterScriptable* scriptable, ClutterScript* script, const gchar* name, const GValue* value) {
-	_tangle_scriptable_set_custom_property(scriptable, script, name, value);
+	TangleObject* object;
+	
+	object = TANGLE_OBJECT(scriptable);
+
+	_tangle_scriptable_set_custom_property(scriptable, script, name, value, &object->priv->template_parameters, object->priv->template_found);
 }
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
@@ -161,15 +130,3 @@ static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
 	iface->parse_custom_node = tangle_object_parse_custom_node;
 	iface->set_custom_property = tangle_object_set_custom_property;
 }
-
-static gboolean check_template_parameters(gpointer user_data) {
-	TangleObject* object;
-	
-	object = TANGLE_OBJECT(user_data);
-	
-	if (object->priv->template && !object->priv->template_applied) {
-		g_critical("Some template (id: %s) parameters in object (id: %s) are missing. Template is not used.", clutter_get_script_id(G_OBJECT(object->priv->template)), clutter_get_script_id(G_OBJECT(object)));
-	}
-	
-	return FALSE;
-}
