@@ -204,7 +204,7 @@ static void tangle_scrollbar_dispose(GObject* object) {
 	
 	scrollbar = TANGLE_SCROLLBAR(object);
 	
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(scrollbar->priv->bar_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT(scrollbar->priv->bar_actor);
 
 	G_OBJECT_CLASS(tangle_scrollbar_parent_class)->dispose(object);
 }
