@@ -26,5 +26,8 @@ void tangle_vault_get(TangleVault* vault, guint n, GType first_type, ...);
 #define tangle_signal_connect_vault(instance, detailed_signal, c_handler, vault) \
 	g_signal_connect_data((instance), (detailed_signal), (c_handler), (vault), (GClosureNotify)tangle_vault_free, (GConnectFlags)0)
 
+#define tangle_signal_connect_vault_flags(instance, detailed_signal, c_handler, vault, flags) \
+	g_signal_connect_data((instance), (detailed_signal), (c_handler), (vault), (GClosureNotify)tangle_vault_free, flags)
+
 #endif
 
