@@ -11,6 +11,8 @@
 #include "tangle-stylable.h"
 #include "tangle-stylesheet.h"
 #include "tangle-misc.h"
+#include "tangle-scroll-action.h"
+#include "tangle-scroll-trick.h"
 
 /**
  * SECTION:tangle-widget
@@ -27,11 +29,11 @@ G_DEFINE_TYPE_WITH_CODE(TangleWidget, tangle_widget, TANGLE_TYPE_ACTOR,
 
 enum {
 	PROP_0,
-	PROP_PARENT,
 	PROP_N_CHILDREN,
 	PROP_CHILREN_READONLY,
 	PROP_LAYOUT,
 	PROP_PADDING,
+	PROP_PADDING_SET,
 	PROP_CHILDREN_ALLOCATION,
 	PROP_BACKGROUND_COLOR,
 	PROP_BACKGROUND_COLOR_SET,
@@ -39,7 +41,9 @@ enum {
 	PROP_FOREGROUND_ACTOR,
 	PROP_PREFER_BACKGROUND_SIZE,
 	PROP_PREFER_BACKGROUND_SIZE_SET,
-	PROP_PICKABLE
+	PROP_PICKABLE,
+	PROP_DESCENDANT_INTERACTING,
+	PROP_DESCENDANT_DRAGGING
 };
 
 enum {
@@ -57,11 +61,17 @@ struct _TangleWidgetPrivate {
 	ClutterActor* background_actor;
 	ClutterActor* foreground_actor;
 	
+	TangleActor* interacting_descendant;
+	TangleActor* dragging_descendant;
+	
 	guint padding_set : 1;
 	guint background_color_set : 1;
 	guint prefer_background_size : 1;
 	guint prefer_background_size_set : 1;
 	guint pickable : 1;
+	
+	guint background_parented : 1;
+	guint foreground_parented : 1;
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -74,6 +84,8 @@ static void change_actor_order(TangleWidget* widget, ClutterActor* actor, Clutte
 static gint compare_actor_depths(gconstpointer a, gconstpointer b);
 static gint compare_actor_if_higher_depth_position(gconstpointer a, gconstpointer b);
 static void force_actor_depth(ClutterActor* actor, ClutterActor* sibling, gboolean before);
+static void on_notify_interacting(GObject* object, GParamSpec* param_spec, gpointer user_data);
+static void on_notify_dragging(GObject* object, GParamSpec* param_spec, gpointer user_data);
 
 ClutterActor* tangle_widget_new() {
 
@@ -85,9 +97,24 @@ ClutterActor* tangle_widget_new_with_layout(TangleLayout* layout) {
 	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_WIDGET, "layout", layout, NULL));
 }
 
-ClutterActor* tangle_widget_new_with_parent(TangleWidget* parent) {
+ClutterActor* tangle_widget_new_scrollable(TangleLayout* layout) {
+	ClutterActor* actor;
+	ClutterAction* action;
+	TangleTrick* trick;
 
-	return CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_WIDGET, "parent", parent, NULL));
+	actor = CLUTTER_ACTOR(g_object_new(TANGLE_TYPE_WIDGET, "layout", layout, NULL));
+
+	clutter_actor_set_clip_to_allocation(actor, TRUE);
+
+	action = tangle_scroll_action_new();
+	clutter_actor_add_action(actor, action);
+
+	trick = tangle_scroll_trick_new();
+	tangle_layout_add_trick(layout, trick);
+	
+	tangle_scroll_action_bind_to_scroll_trick(TANGLE_SCROLL_ACTION(action), TANGLE_SCROLL_TRICK(trick));
+	
+	return actor;
 }
 
 
@@ -275,7 +302,7 @@ void tangle_widget_set_background_actor(TangleWidget* widget, ClutterActor* acto
 	ClutterActorBox actor_box;
 
 	if (widget->priv->background_actor != actor) {
-		if (widget->priv->background_actor) {
+		if (widget->priv->background_actor && widget->priv->background_parented) {
 			clutter_actor_unparent(widget->priv->background_actor);
 		}
 		widget->priv->background_actor = actor;
@@ -286,6 +313,7 @@ void tangle_widget_set_background_actor(TangleWidget* widget, ClutterActor* acto
 			clutter_actor_push_internal(CLUTTER_ACTOR(widget));
 			clutter_actor_set_parent(actor, CLUTTER_ACTOR(widget));	
 			clutter_actor_pop_internal(CLUTTER_ACTOR(widget));
+			widget->priv->background_parented = TRUE;
 		}
 		
 		g_object_notify(G_OBJECT(widget), "background-actor");
@@ -307,11 +335,15 @@ void tangle_widget_change_background_actor(TangleWidget* widget, ClutterActor* a
 	ClutterActorBox actor_box;
 
 	if (widget->priv->background_actor != actor) {
+		if (widget->priv->background_actor && widget->priv->background_parented) {
+			clutter_actor_unparent(widget->priv->background_actor);
+		}
 		widget->priv->background_actor = actor;
 		
 		if (actor) {
 			tangle_actor_get_aligned_allocation(TANGLE_ACTOR(widget), &actor_box);
 			clutter_actor_allocate(actor, &actor_box, CLUTTER_ABSOLUTE_ORIGIN_CHANGED);
+			widget->priv->background_parented = FALSE;
 		}
 		
 		g_object_notify(G_OBJECT(widget), "background-actor");
@@ -328,8 +360,9 @@ void tangle_widget_set_foreground_actor(TangleWidget* widget, ClutterActor* acto
 	ClutterActorBox actor_box;
 
 	if (widget->priv->foreground_actor != actor) {
-		if (widget->priv->foreground_actor) {
+		if (widget->priv->foreground_actor && widget->priv->foreground_parented) {
 			clutter_actor_unparent(widget->priv->foreground_actor);
+			widget->priv->foreground_parented = FALSE;
 		}
 		widget->priv->foreground_actor = actor;
 		
@@ -339,6 +372,7 @@ void tangle_widget_set_foreground_actor(TangleWidget* widget, ClutterActor* acto
 			clutter_actor_push_internal(CLUTTER_ACTOR(widget));
 			clutter_actor_set_parent(actor, CLUTTER_ACTOR(widget));	
 			clutter_actor_pop_internal(CLUTTER_ACTOR(widget));
+			widget->priv->foreground_parented = TRUE;
 		}
 		
 		g_object_notify(G_OBJECT(widget), "foreground-actor");
@@ -402,6 +436,17 @@ void tangle_widget_get_real_preferred_height(TangleWidget* widget, gfloat for_wi
 	tangle_widget_get_preferred_height(TANGLE_ACTOR(widget), for_width, interacting, min_height_p, natural_height_p, max_height_p);
 }
 
+gboolean tangle_widget_get_descendant_interacting(TangleWidget* widget) {
+
+	return widget->priv->interacting_descendant != NULL;
+}
+
+gboolean tangle_widget_get_descendant_dragging(TangleWidget* widget) {
+
+	return widget->priv->dragging_descendant != NULL;
+}
+
+
 #define ABS_F(x) ((x) < 0 ? -(x) : (x))
 
 void tangle_widget_clamp_child_boundaries(TangleWidget* widget, TangleClamp* clamp) {
@@ -563,7 +608,6 @@ static void tangle_widget_get_preferred_width(TangleActor* actor, gfloat for_hei
 			}
 		}
 
-
 		if (widget->priv->padding_set) {
 			if (*min_width_p) {
 				*min_width_p += widget->priv->padding.left + widget->priv->padding.right;
@@ -710,18 +754,28 @@ static void tangle_widget_map(ClutterActor* actor) {
 
 	CLUTTER_ACTOR_CLASS(tangle_widget_parent_class)->map(actor);
 
-	if (widget->priv->background_actor) {
+	if (widget->priv->background_actor && widget->priv->background_parented) {
 		clutter_actor_map(widget->priv->background_actor);
 	}
-	if (widget->priv->foreground_actor) {
+	if (widget->priv->foreground_actor && widget->priv->background_parented) {
 		clutter_actor_map(widget->priv->foreground_actor);
 	}
 }
 
-static void tangle_widget_handle_ancestor_event(TangleActor* actor, TangleEvent* event) {
-	tangle_widget_foreach_type(TANGLE_WIDGET(actor), TANGLE_TYPE_ACTOR, CLUTTER_CALLBACK(tangle_actor_handle_ancestor_event), event);
+static void tangle_widget_unmap(ClutterActor* actor) {
+	TangleWidget* widget;
+	TangleStylesheet* stylesheet;
 
-	TANGLE_ACTOR_CLASS(tangle_widget_parent_class)->handle_ancestor_event(actor, event);
+	widget = TANGLE_WIDGET(actor);
+
+	if (widget->priv->background_actor && widget->priv->background_parented) {
+		clutter_actor_unmap(widget->priv->background_actor);
+	}
+	if (widget->priv->foreground_actor && widget->priv->background_parented) {
+		clutter_actor_unmap(widget->priv->foreground_actor);
+	}
+
+	CLUTTER_ACTOR_CLASS(tangle_widget_parent_class)->unmap(actor);
 }
 
 static void tangle_widget_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
@@ -733,6 +787,12 @@ static void tangle_widget_set_property(GObject* object, guint prop_id, const GVa
 		case PROP_LAYOUT:
 			tangle_widget_set_layout(widget, g_value_get_object(value));
 			break;
+		case PROP_PADDING:
+			tangle_widget_set_padding(widget, TANGLE_SPACING(g_value_get_boxed(value)));
+			break;
+		case PROP_PADDING_SET:
+			tangle_widget_set_padding_set(widget, g_value_get_boolean(value));
+			break;
 		case PROP_BACKGROUND_COLOR:
 			tangle_widget_set_background_color(widget, clutter_value_get_color(value));
 			break;
@@ -763,9 +823,15 @@ static void tangle_widget_get_property(GObject* object, guint prop_id, GValue* v
 	widget = TANGLE_WIDGET(object);
 
         switch (prop_id) {
-		PROP_LAYOUT:
+		case PROP_LAYOUT:
 			g_value_set_object(value, tangle_widget_get_layout(widget));
 			break;
+		case PROP_PADDING:
+			g_value_set_boxed(value, &widget->priv->padding);
+			break;
+		case PROP_PADDING_SET:
+			g_value_set_boolean(value, widget->priv->padding_set);
+			break;
 		case PROP_CHILDREN_ALLOCATION:
 			g_value_set_boxed(value, &widget->priv->children_allocation);
 			break;
@@ -787,19 +853,33 @@ static void tangle_widget_get_property(GObject* object, guint prop_id, GValue* v
 		case PROP_PICKABLE:
 			g_value_set_boolean(value, widget->priv->pickable);
 			break;
+		case PROP_DESCENDANT_INTERACTING:
+			g_value_set_boolean(value, widget->priv->interacting_descendant != NULL);
+			break;
+		case PROP_DESCENDANT_DRAGGING:
+			g_value_set_boolean(value, widget->priv->dragging_descendant != NULL);
+			break;
 	        default:
 		        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
 		        break;
         }
 }
 
+static void unparent_child(ClutterActor* actor, TangleWidget* widget) {
+	clutter_actor_unparent(actor);
+	if (TANGLE_IS_ACTOR(actor)) {
+		g_signal_handlers_disconnect_by_func(actor, G_CALLBACK(on_notify_interacting), widget);
+		g_signal_handlers_disconnect_by_func(actor, G_CALLBACK(on_notify_dragging), widget);
+	}
+}
+
 static void tangle_widget_dispose(GObject* object) {
 	TangleWidget* widget;
 	
 	widget = TANGLE_WIDGET(object);
 	
 	if (widget->priv->children) {
-		g_list_foreach(widget->priv->children, (GFunc)clutter_actor_unparent, NULL);
+		g_list_foreach(widget->priv->children, (GFunc)unparent_child, widget);
 		g_list_free(widget->priv->children);
 		widget->priv->children = NULL;
 	}
@@ -811,14 +891,11 @@ static void tangle_widget_dispose(GObject* object) {
 	}
 	
 	TANGLE_UNREF_AND_NULLIFY_OBJECT(widget->priv->stylesheet);
-
-	if (widget->priv->background_actor) {
-		clutter_actor_unparent(widget->priv->background_actor);
-		widget->priv->background_actor = NULL;
+	if (widget->priv->background_parented) {
+		TANGLE_UNPARENT_AND_NULLIFY_ACTOR(widget->priv->background_actor);
 	}
-	if (widget->priv->foreground_actor) {
-		clutter_actor_unparent(widget->priv->foreground_actor);
-		widget->priv->foreground_actor = NULL;
+	if (widget->priv->foreground_parented) {
+		TANGLE_UNPARENT_AND_NULLIFY_ACTOR(widget->priv->foreground_actor);
 	}
 
 	G_OBJECT_CLASS(tangle_widget_parent_class)->dispose(object);
@@ -837,10 +914,10 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	clutter_actor_class->pick = tangle_widget_pick;
 	clutter_actor_class->allocate = tangle_widget_allocate;
 	clutter_actor_class->map = tangle_widget_map;
+	clutter_actor_class->unmap = tangle_widget_unmap;
 
 	actor_class->get_preferred_width = tangle_widget_get_preferred_width;
 	actor_class->get_preferred_height = tangle_widget_get_preferred_height;
-	actor_class->handle_ancestor_event = tangle_widget_handle_ancestor_event;
 
 	widget_class->paint_children = tangle_widget_paint_children;
 	widget_class->allocate_children = tangle_widget_allocate_children;
@@ -854,6 +931,28 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	                                "The layout that is used to position children",
 	                                TANGLE_TYPE_LAYOUT,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:padding:
+	 *
+	 * The padding around the child actors. This does not affect to background or foreground.
+	 */
+	g_object_class_install_property(gobject_class, PROP_PADDING,
+	                                g_param_spec_boxed("padding",
+	                                                    "Padding",
+	                                                    "The padding around the child actors",
+	                                                    TANGLE_TYPE_SPACING,
+	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleActor:padding-set:
+	 *
+	 * Whether the :padding is set.
+	 */
+	g_object_class_install_property(gobject_class, PROP_PADDING_SET,
+	                                g_param_spec_boolean("padding-set",
+	                                                    "Padding set",
+	                                                    "Whether the padding is set",
+	                                                    FALSE,
+	                                                    G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 	/**
 	 * TangleWidget:children-allocation:
 	 */
@@ -945,6 +1044,28 @@ static void tangle_widget_class_init(TangleWidgetClass* widget_class) {
 	                                "Whether the widget is pickable or not",
 	                                TRUE,
 	                                G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:descendant-interacting:
+	 *
+	 * Whether an descendant #TangleActor is interacting.
+	 */
+	g_object_class_install_property(gobject_class, PROP_DESCENDANT_INTERACTING,
+	                                g_param_spec_boolean("descendant-interacting",
+	                                "Descendant interacting",
+	                                "Whether an descendant actor is interacting",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
+	/**
+	 * TangleWidget:descendant-dragging:
+	 *
+	 * Whether an descendant #TangleActor is under dragging.
+	 */
+	g_object_class_install_property(gobject_class, PROP_DESCENDANT_DRAGGING,
+	                                g_param_spec_boolean("descendant-dragging",
+	                                "Descendant dragging",
+	                                "Whether an descendant actor is under dragging",
+	                                FALSE,
+	                                G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |G_PARAM_STATIC_BLURB));
 
 	signals[PAINT_CHILDREN] = g_signal_new("paint-children",
 	                                       G_TYPE_FROM_CLASS(gobject_class),
@@ -1002,6 +1123,13 @@ static void tangle_widget_add_impl(TangleWidget* widget, ClutterActor* actor, Cl
 	if (widget->priv->stylesheet) {
 		tangle_stylesheet_apply(widget->priv->stylesheet, G_OBJECT(actor));
 	}
+	
+	if (TANGLE_IS_ACTOR(actor)) {
+		g_signal_connect(actor, "notify::interacting", G_CALLBACK(on_notify_interacting), widget);
+		g_signal_connect(actor, "notify::descendant-interacting", G_CALLBACK(on_notify_interacting), widget);
+		g_signal_connect(actor, "notify::dragging", G_CALLBACK(on_notify_dragging), widget);
+		g_signal_connect(actor, "notify::descendant-dragging", G_CALLBACK(on_notify_dragging), widget);
+	}
 
 	clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
 
@@ -1014,23 +1142,38 @@ static void tangle_widget_real_add(ClutterContainer* widget, ClutterActor* actor
 	tangle_widget_add(TANGLE_WIDGET(widget), actor, NULL);
 }
 
-static void tangle_widget_remove(ClutterContainer* widget, ClutterActor* actor) {
+static void tangle_widget_remove(ClutterContainer* container, ClutterActor* actor) {
+	TangleWidget* widget;
 	gboolean was_visible;
 
-	g_return_if_fail(TANGLE_IS_WIDGET(widget));
+	g_return_if_fail(TANGLE_IS_WIDGET(container));
 	g_return_if_fail(CLUTTER_IS_ACTOR(actor));
-	g_return_if_fail(clutter_actor_get_parent(actor) == CLUTTER_ACTOR(widget));
+	g_return_if_fail(clutter_actor_get_parent(actor) == CLUTTER_ACTOR(container));
+
+	widget = TANGLE_WIDGET(container);
 
 	g_object_ref(actor);
+	
+	if (CLUTTER_ACTOR(widget->priv->interacting_descendant) == actor) {
+		widget->priv->interacting_descendant = NULL;
+		g_object_notify(G_OBJECT(widget), "descendant-interacting");
+	}
+	if (CLUTTER_ACTOR(widget->priv->dragging_descendant) == actor) {
+		widget->priv->dragging_descendant = NULL;
+		g_object_notify(G_OBJECT(widget), "descendant-dragging");
+	}
 
 	was_visible = CLUTTER_ACTOR_IS_VISIBLE(actor);
 
-	TANGLE_WIDGET(widget)->priv->children = g_list_remove(TANGLE_WIDGET(widget)->priv->children, actor);
-	if (TANGLE_WIDGET(widget)->priv->stylesheet) {
-		tangle_stylesheet_unapply(TANGLE_WIDGET(widget)->priv->stylesheet, G_OBJECT(actor));
+	widget->priv->children = g_list_remove(widget->priv->children, actor);
+	if (widget->priv->stylesheet) {
+		tangle_stylesheet_unapply(widget->priv->stylesheet, G_OBJECT(actor));
 	}
 	clutter_actor_unparent(actor);
 	
+	g_signal_handlers_disconnect_by_func(actor, G_CALLBACK(on_notify_interacting), widget);
+	g_signal_handlers_disconnect_by_func(actor, G_CALLBACK(on_notify_dragging), widget);
+	
 	if (was_visible) {
 		clutter_actor_queue_relayout(CLUTTER_ACTOR(widget));
 	}
@@ -1100,8 +1243,8 @@ static gboolean tangle_widget_unapply_stylesheet(TangleStylable* stylable, Tangl
 	widget = TANGLE_WIDGET(stylable);
 	if (widget->priv->stylesheet == stylesheet) {
 		tangle_widget_foreach_swapped(widget, TANGLE_SWAPPED_CALLBACK(tangle_stylesheet_unapply), stylesheet);
-		widget->priv->stylesheet = NULL;
 		g_object_unref(widget->priv->stylesheet);
+		widget->priv->stylesheet = NULL;
 	}
 
 	return FALSE;
@@ -1222,3 +1365,74 @@ static void force_actor_depth(ClutterActor* actor, ClutterActor* sibling, gboole
 		clutter_actor_set_depth(actor, depth_sibling);
 	}
 }
+
+static void stop_interacting(TangleActor* actor) {
+	TangleWidget* widget;
+
+	if (actor) {
+		if (TANGLE_IS_WIDGET(actor)) {
+			widget = TANGLE_WIDGET(actor);
+			stop_interacting(widget->priv->interacting_descendant);
+			widget->priv->interacting_descendant = NULL;
+			g_object_notify(G_OBJECT(widget), "descendant-interacting");
+		}
+		tangle_actor_set_interacting(actor, FALSE);
+	}
+}
+
+static void on_notify_interacting(GObject* object, GParamSpec* param_spec, gpointer user_data) {
+	TangleWidget* widget;
+	TangleActor* actor;
+	
+	widget = TANGLE_WIDGET(user_data);
+	actor = TANGLE_ACTOR(object);
+	
+	if (tangle_actor_get_interacting(actor) || (TANGLE_IS_WIDGET(actor) && tangle_widget_get_descendant_interacting(TANGLE_WIDGET(actor)))) {
+		if (widget->priv->interacting_descendant != actor) {
+			stop_interacting(widget->priv->interacting_descendant);
+			widget->priv->interacting_descendant = actor;
+			g_object_notify(G_OBJECT(widget), "descendant-interacting");
+		}
+	} else {
+		if (widget->priv->interacting_descendant == actor) {
+			widget->priv->interacting_descendant = NULL;
+			g_object_notify(G_OBJECT(widget), "descendant-interacting");
+		}
+	}
+}
+
+static void stop_dragging(TangleActor* actor) {
+	TangleWidget* widget;
+
+	if (actor) {
+		if (TANGLE_IS_WIDGET(actor)) {
+			widget = TANGLE_WIDGET(actor);
+			stop_dragging(widget->priv->dragging_descendant);
+			widget->priv->dragging_descendant = NULL;
+			g_object_notify(G_OBJECT(widget), "descendant-dragging");
+		}
+		tangle_actor_set_dragging(actor, FALSE);
+	}
+}
+
+static void on_notify_dragging(GObject* object, GParamSpec* param_spec, gpointer user_data) {
+	TangleWidget* widget;
+	TangleActor* actor;
+	
+	widget = TANGLE_WIDGET(user_data);
+
+	actor = TANGLE_ACTOR(object);
+	
+	if (tangle_actor_get_dragging(actor) || (TANGLE_IS_WIDGET(actor) && tangle_widget_get_descendant_dragging(TANGLE_WIDGET(actor)))) {
+		if (widget->priv->dragging_descendant != actor) {
+			stop_dragging(widget->priv->dragging_descendant);
+			widget->priv->dragging_descendant = actor;
+			g_object_notify(G_OBJECT(widget), "descendant-dragging");
+		}
+	} else {
+		if (widget->priv->dragging_descendant == actor) {
+			widget->priv->dragging_descendant = NULL;
+			g_object_notify(G_OBJECT(widget), "descendant-dragging");
+		}
+	}
+}
