@@ -143,18 +143,6 @@ static void tangle_wrapper_actor_get_preferred_height(TangleActor* actor, gfloat
 	}
 }
 
-static void tangle_wrapper_actor_handle_ancestor_event(TangleActor* actor, TangleEvent* event) {
-	TangleWrapperActor* wrapper_actor;
-	
-	TANGLE_ACTOR_CLASS(tangle_wrapper_actor_parent_class)->handle_ancestor_event(actor, event);
-	
-	wrapper_actor = TANGLE_WRAPPER_ACTOR(actor);
-	if (TANGLE_IS_ACTOR(wrapper_actor->priv->wrapped)) {
-		tangle_actor_handle_ancestor_event(TANGLE_ACTOR(wrapper_actor->priv->wrapped), event);
-	}
-}
-	
-
 static void tangle_wrapper_actor_allocate_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped, const ClutterActorBox* box, ClutterAllocationFlags flags) {
 	clutter_actor_allocate(wrapped, box, flags);
 }
@@ -280,7 +268,6 @@ static void tangle_wrapper_actor_class_init(TangleWrapperActorClass* wrapper_act
 	
 	actor_class->get_preferred_width = tangle_wrapper_actor_get_preferred_width;
 	actor_class->get_preferred_height = tangle_wrapper_actor_get_preferred_height;
-	actor_class->handle_ancestor_event = tangle_wrapper_actor_handle_ancestor_event;
 
 	wrapper_actor_class->allocate_wrapped = tangle_wrapper_actor_allocate_wrapped;
 	wrapper_actor_class->paint_wrapped = tangle_wrapper_actor_paint_wrapped;
